﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using CBIOS4NET;
using CBIOS4NET.Protection;
using System.Security.Cryptography;

namespace MARX.Security
{
    public interface IMarxProtection
    {
        void Start();
        event EventHandler<ProtectionMessage> validationFailed;
        event EventHandler<ProtectionMessage> validationSucceed;
    }

    public class ProtectionMessage : EventArgs
    {
        public enum ErrorCode : ushort
        {
            None = 0,
            BoxNotFound = 1,
            NetworkConnectionFailed = 2,
            LicenseExpired = 3,
            CBIOSError = 4,
            ValidationFailed = 5,
            LicenseFileNotFound = 6,
            Unknown = 100
        }

        public ProtectionMessage()
        {
            Message = "";
            Code = ErrorCode.None;
        }

        public string Message;

        public ErrorCode Code;
    }

    public abstract class ProtectionBase : IMarxProtection
    {
        protected ProtectionMessage.ErrorCode _lastError = ProtectionMessage.ErrorCode.None;

        public event EventHandler<ProtectionMessage> validationFailed;
        public event EventHandler<ProtectionMessage> validationSucceed;
        protected void Notify(ProtectionMessage.ErrorCode code, string message = "")
        {
            if (code != ProtectionMessage.ErrorCode.None
                && _lastError == ProtectionMessage.ErrorCode.None)
            {
                validationFailed.Invoke(this, new ProtectionMessage()
                {
                    Message = message,
                    Code = code
                });
            }
            else if (code == ProtectionMessage.ErrorCode.None
                && _lastError != ProtectionMessage.ErrorCode.None)
            {
                validationSucceed.Invoke(this, new ProtectionMessage()
                {
                    Code = code
                });
            }
            _lastError = code;
        }

        public abstract void Start();
    }

    public class ProtectionLicenseFile : ProtectionBase, IDisposable
    {
        SmarxLicense smarxLicense = null;

        Task _task;

        string _licenseFile;

        public ProtectionLicenseFile(string licenseFile)
        {
            _licenseFile = licenseFile;
        }

        ~ProtectionLicenseFile()
        {
            Dispose();
        }

        public override void Start()
        {
            if (Init(_licenseFile))
            {
                _task = new Task(() =>
                {
                    while (true)
                    {
                        LicenseValidate();
                        Thread.Sleep(4000);
                    }
                });
                _task.Start();
            }
        }

        void LicenseValidate()
        {
            try
            {
                if (null == smarxLicense) Init(_licenseFile);

                smarxLicense.Validate();

                Notify(ProtectionMessage.ErrorCode.None);
            }
            catch (Exception ex)
            {
                string s = ex.Message;
                if (null != ex.InnerException)
                    s += Environment.NewLine + Environment.NewLine + ex.InnerException.Message;
                Notify(ProtectionMessage.ErrorCode.ValidationFailed, s);
            }
        }

        public void Dispose()
        {
            smarxLicense.Dispose();
        }

        bool Init(string s)
        {
            try
            {
                smarxLicense = new SmarxLicense();
                smarxLicense.LoadFile(s);

                return true;
            }
            catch (Exception)
            {
                Notify(ProtectionMessage.ErrorCode.LicenseFileNotFound, "License file is not found");
                smarxLicense = null;
            }

            return false;
        }
    }

    public abstract class ManualAPIProtection : ProtectionBase, IDisposable
    {
        // Check [PPK]\SmarxOS\API\Win\Samples\Security\ProtectUPW sample
        // for an example how to generate RSA key and protect UPW 
        static byte[] RsaKeyModulus = new byte[]
            {
            0x20, 0x59, 0xe1, 0x3a, 0xfd, 0x97, 0xd1, 0x9d, 0xd6, 0x9e, 0x22, 0x75, 0x17, 0x1f, 0x96, 0x1b,
            0xae, 0x96, 0x6c, 0x81, 0x1e, 0x92, 0x6f, 0x42, 0x15, 0x05, 0x71, 0xe3, 0x3e, 0x82, 0x2a, 0x2a,
            0x28, 0xf4, 0x82, 0x96, 0xf9, 0x31, 0x14, 0x6a, 0xb2, 0x94, 0x7c, 0x98, 0x5b, 0xc9, 0xff, 0xd4,
            0x45, 0xeb, 0x5e, 0x3d, 0xfb, 0xef, 0x37, 0x25, 0x9b, 0x9b, 0x84, 0x61, 0xe9, 0x8e, 0xce, 0xa3
            };
        static byte[] RsaKeyExponent = new byte[]
            {
            0xec, 0xf9, 0xc4, 0xe0, 0x5b, 0x9b, 0x74, 0x96, 0x26, 0x87, 0xf7, 0x2f, 0xaa, 0x0c, 0x33, 0x5c,
            0x68, 0x25, 0xfc, 0x98, 0x7a, 0x7c, 0x50, 0x42, 0x4f, 0xcd, 0xef, 0x48, 0x3d, 0x49, 0x70, 0x38,
            0x8c, 0xfc, 0x17, 0x81, 0x22, 0xa1, 0x21, 0xa8, 0x57, 0xb0, 0x78, 0xba, 0x5a, 0x5e, 0x96, 0xa8,
            0x24, 0x3e, 0xe9, 0x87, 0xf0, 0xb2, 0x55, 0xbf, 0x37, 0x11, 0xf6, 0x36, 0x73, 0xfc, 0x92, 0x05
            };  


        protected CryptoboxManager _cbManager = null;
        protected Cryptobox _cBox = null;
        protected Task _task = null;
        protected IObserver<ProtectionMessage> _observer;

        protected ushort _appID = 0;
        int _boxReconnectTimeout = 2000;
        int _validationTimeout = 2000;

        static uint TEOSDO_EXPIRATION_DATE_TIME_DOID = 1001;
        static uint TEOSDO_EXPIRATION_DATE_TIME_OFFSET = 0;

        static uint TEOSDO_COUNTER_DOID = 1002;
        static uint TEOSDO_COUNTER_OFFSET = 20;

        static RAM TEOSDO_UPW_RAM = RAM.RAM3;
        static uint TEOSDO_UPW_MEMORY_DOID = 3001;
        static uint TEOSDO_UPW_MEMORY_OFFSET = 0;

        static RAM TEOSDO_AES_RAM = RAM.RAM3;
        static uint TEOSDO_AES_MEMORY_DOID = 3002;


        static uint TEOSDO_CDO_SIGNATURE_SIZE = 12;
        static uint CBIOS_AES_KEY_LEN = 16;

        static uint ENCRYPTED_UPW_LENGTH = 64;

        protected static bool isCounterChecked = false;
        protected static bool isRunsLeft = false;


        public ManualAPIProtection(ushort AppID, int boxReconnectTimeout = 2000, int validationTimeout = 5000)
        {
            _appID = AppID;
            _boxReconnectTimeout = boxReconnectTimeout;
            _validationTimeout = validationTimeout;
        }

        ~ManualAPIProtection()
        {
            Dispose();
        }

        protected bool ConnectBox(ushort appID)
        {
            if (_cbManager == null)
            {
                return false;
            }
            BoxHandle[] boxes = _cbManager.ScanBoxes();
            BoxHandle selectedbox = null;

            foreach (BoxHandle box in boxes)
            {
                foreach (AppInfo app in box.Applications)
                {
                    if (app.AppId == appID)
                    {
                        selectedbox = box;
                        break;
                    }
                }
                if (selectedbox != null)
                {
                    break;
                }
            }

            if (selectedbox != null)
            {
                _cBox = _cbManager.OpenByIndex(selectedbox.Index, appID);
            }

            return _cBox == null ? false : true;

        }

        protected bool ProtectedUPWLogin()
        {
            uint UPWDoId = TEOSDO_UPW_MEMORY_DOID;
            uint UPWDoOffset = TEOSDO_UPW_MEMORY_OFFSET;
            uint AESDoId = TEOSDO_AES_MEMORY_DOID;

            bool result = false;

            Partition p = new Partition();
            CryptoDataObjectMemory doUPWMemory = new CryptoDataObjectMemory(UPWDoId, TEOSDO_UPW_RAM, UPWDoOffset);
            CryptoDataObjectMemory doAESMemory = new CryptoDataObjectMemory(AESDoId, TEOSDO_AES_RAM, ENCRYPTED_UPW_LENGTH + TEOSDO_CDO_SIGNATURE_SIZE);

            try
            {
                p.Add(doUPWMemory);
                p.Add(doAESMemory);
                if (p.Connect(_cBox))
                {
                    byte[] UPW = doUPWMemory.Get();

                    byte[] AES = doAESMemory.Get();
                    byte[] AESKey = new byte[CBIOS_AES_KEY_LEN];
                    byte[] AESIV = new byte[CBIOS_AES_KEY_LEN];

                    Array.Copy(AES, 0, AESKey, 0, CBIOS_AES_KEY_LEN);
                    Array.Copy(AES, CBIOS_AES_KEY_LEN, AESIV, 0, CBIOS_AES_KEY_LEN);
                    _cBox.SetKeySession(new RijndaelCryptKey(AESKey, AESIV));

                    CBU2RSAKey RSA_PUBLIC_KEY = new CBU2RSAKey(_cBox.CryptSession(RsaKeyModulus), _cBox.CryptSession(RsaKeyExponent));
                    _cBox.SetKeySession(new RijndaelCryptKey(
                        new byte[] { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
                        new byte[] { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }
                        ));
                    byte[] decryptedUPW = CBU2RSA.DecryptRSA(RSA_PUBLIC_KEY, CBU2RSAKeyMode.PUBL_KEY | CBU2RSAKeyMode.RSAREF_PADDING, UPW);

                    result = _cBox.Login(CryptoboxLoginType.User, decryptedUPW);
                    decryptedUPW = Enumerable.Repeat((byte)0, decryptedUPW.Length).ToArray();
                    result = true;
                }
            }
            catch (CBIOSException ex)
            {
                Notify(ProtectionMessage.ErrorCode.CBIOSError, "CBIOS error: " + ex.ErrorCode);
            }
            catch(Exception ex)
            {
                Notify(ProtectionMessage.ErrorCode.Unknown, "Unknown error: " + ex.Message);
            }
            finally
            {
                doAESMemory.Dispose();
                doUPWMemory.Dispose();
                p.Clear();
                p.Dispose();
            }

            return result;
        }

        protected bool ValidateExpDate()
        {
            uint ExpDateDoId = TEOSDO_EXPIRATION_DATE_TIME_DOID;
            uint ExpDateDoOffset = TEOSDO_EXPIRATION_DATE_TIME_OFFSET;
            bool result = false;

            Partition p = new Partition();
            CryptoDataObjectExpirationDateTime doExpDate = new CryptoDataObjectExpirationDateTime(ExpDateDoId, RAM.RAM1, ExpDateDoOffset);

            try
            {
                    p.Add(doExpDate);
                    if (p.Connect(_cBox))
                    {
                        if (!doExpDate.Verify())
                        {
                            Notify(ProtectionMessage.ErrorCode.LicenseExpired, "License has expired");
                        }
                        else
                        {
                            Notify(ProtectionMessage.ErrorCode.None);
                            result = true;
                        }
                    }
                    else
                    {
                        Notify(ProtectionMessage.ErrorCode.BoxNotFound, "Box not found");
                    }


            }
            catch (CBIOSException ex)
            {
                Notify(ProtectionMessage.ErrorCode.CBIOSError, "CBIOS error: " + ex.ErrorCode);
            }
            finally
            {
                p.Clear();
                p.Dispose();
                doExpDate.Dispose();
            }

            return result;
        }

        protected bool ValidateRunCounter()
        {
            uint RunCounterDoId = TEOSDO_COUNTER_DOID;
            uint RunCounterDoOffset = TEOSDO_COUNTER_OFFSET;
            bool result = false;

            Partition p = new Partition();
            CryptoDataObjectRunCounter doRunCounter = new CryptoDataObjectRunCounter(RunCounterDoId, RAM.RAM1, RunCounterDoOffset);

            try
            {
                p.Add(doRunCounter);
                if (p.Connect(_cBox))
                {
                    doRunCounter.Dec();
                    result = true;
                }
                else
                {
                    Notify(ProtectionMessage.ErrorCode.BoxNotFound, "Box not found");
                }

            }
            catch (CBIOSException ex)
            {
                if (ex.ErrorCode == CBIOSErrorCode.TEOSERR_NO_USAGE_COUNTER)
                {
                    Notify(ProtectionMessage.ErrorCode.CBIOSError, "No runs left");
                }
                else
                {
                    Notify(ProtectionMessage.ErrorCode.CBIOSError, "CBIOS error: " + ex.ErrorCode);
                }
            }
            finally
            {
                p.Clear();
                p.Dispose();
                doRunCounter.Dispose();
            }

            return result;
        }

        protected void StartValidation()
        {
            while (true)
            {
                if (!ConnectBox(_appID))
                {
                    Notify(ProtectionMessage.ErrorCode.BoxNotFound, "Box not found");
                    Thread.Sleep(_boxReconnectTimeout);
                    continue;
                }
                break;
            }

            if (!ProtectedUPWLogin())
            {
                Notify(ProtectionMessage.ErrorCode.CBIOSError, "CBU Login failed");
                return;
            }
            else
            {
                if (!isCounterChecked)
                {
                    isCounterChecked = true;
                    if (ValidateRunCounter())
                    {
                        isRunsLeft = true;
                    }
                    else
                    {
                        Notify(ProtectionMessage.ErrorCode.LicenseExpired, "No runs left");
                        _cBox.Logout();
                    }
                    
                }

                if (!isRunsLeft)
                    return;

                while (true)
                {
                    try
                    {
                        if (_cBox.CheckBox())
                        {
                            ValidateExpDate();
                        }
                        else
                        {
                            Notify(ProtectionMessage.ErrorCode.BoxNotFound, "CBU not found");
                        }

                        if (_lastError == ProtectionMessage.ErrorCode.BoxNotFound)
                        {
                            break;
                        }

                    }
                    catch (CBIOSException ex)
                    {
                        Notify(ProtectionMessage.ErrorCode.CBIOSError, "CBIOS error: " + ex.ErrorCode);
                        break;
                    }
                    Thread.Sleep(_validationTimeout);
                }

                if (_lastError == ProtectionMessage.ErrorCode.LicenseExpired)
                    _cBox.Logout();
            }
        }

        public void Dispose()
        {
            if (_cbManager != null)
                _cbManager.Dispose();
        }



    }

    internal class NetworkAppProtection : ManualAPIProtection
    {
        static int SCAN_NETWORK_TIMEOUT = 2000;
        int _srvReconnectTimeout = 2000;
        bool _isReconnect = false;

        string _hostName;
        ushort _port;
        public NetworkAppProtection(ushort AppID, string host = "localhost", ushort port = 8765, 
            bool reconnect = true, int srvReconnectTimeout = 2000) : base(AppID)
        {
            _hostName = host;
            _port = port;
            _isReconnect = reconnect;
            _srvReconnectTimeout = srvReconnectTimeout;
        }

        public NetworkAppProtection(ushort AppID,
            bool reconnect, int srvReconnectTimeout = 2000) : base(AppID)
        {
            _isReconnect = reconnect;
            _srvReconnectTimeout = srvReconnectTimeout;
        }

        public override void Start()
        {
            _task = new Task(() =>
            {
                while (true)
                {
                    while (true)
                    {
                        if (!ConnectToServer())
                        {
                            Notify(ProtectionMessage.ErrorCode.NetworkConnectionFailed, "Failed to connect to a CBIOS Network Server");                
                            if (!_isReconnect)
                            {
                                return;
                            }
                            Thread.Sleep(_srvReconnectTimeout);
                            continue;
                        }
                        break;
                    }

                    StartValidation();                    

                    _cbManager.Dispose();
                    _cbManager = null;
                    _cBox.Dispose();
                    _cBox = null;
                }
            });
            _task.Start();
        }

        public bool ConnectToServer()
        {
            NetworkCBManager netCBM = new NetworkCBManager();

            if (_hostName != null && _port != null)
            {
                try
                {
                    netCBM.Connect(_hostName, _port);
                }
                catch (CBIOSException ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
            else
            {
                ServerInfo[] servers = netCBM.ScanNetwork(SCAN_NETWORK_TIMEOUT);


                foreach (ServerInfo srv in servers)
                {
                    try
                    {
                        netCBM.Connect(srv.Name, (ushort)srv.ScanPort);
                        _cbManager = netCBM;
                        break;
                    }
                    catch (CBIOSException ex)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }
            }

            return _cbManager == null ? false : true;
        }
    }

    internal class ProtectionLocal : ManualAPIProtection
    {
        public ProtectionLocal(ushort AppID) : base(AppID)
        {
            _cbManager = new LocalCBManager();
        }
        ~ProtectionLocal()
        {
            _cbManager.Dispose();
            _cbManager = null;
        }

        public override void Start()
        {
            _task = new Task(() =>
            {
                while (true)
                {
                    StartValidation();

                    _cBox.Dispose();
                    _cBox = null;
                }
            });

            _task.Start();
        }

    }
}
