{*****************************************************************************
Copyright:   : MARX CryptoTech LP
Project      : CBIOS: CRYPTO-BOX Input/Output System API test
Created      : 01Jul2003
Tools        : Delphi 5 - XE
Description  : Utility functions for CBIOS API test
******************************************************************************
VER         DATE        WHO   DETAIL
******************************************************************************
1.00        01Jul2003   OK    Initial version
1.0.3.701   01Jul2003   OK
1.0.3.703   03Jul2003   OK    Complete test
1.0.3.925   25Sep2003   OK    Changed to match changes in C-based CBIOS
1.0.4.429   29Apr2004   OK    Changed to sample
1.0.6.715   15Jul2006   GV    Rebuilding with new (local+network) CBIOS.dcu
1.5.9.0922  22Sep2009   GV    Sample revision for CBIOS 1.5
1.5.10.0322 22Mar2010   GV    ByteArrayToString/2:"String to ANSIString" changed for D2007-2010
1.6.10.1023 23Dec2010   GV    Sample improvements
1.7         22Oct2012   VG    Sample revision
1.8         27Feb2014   VG    New functions added
*****************************************************************************}

Unit testutil;

Interface
Uses Windows, cbios, testglobals, SysUtils, System.Classes, System.Math;

Procedure PrintError(ErrorText: string);
Procedure PrintCBIOSErrorCode(Error: DWORD);
Procedure PrintDOErrorCode(Error: DWORD);
Procedure PrintOK; overload;
Procedure PrintOK(s: string); overload;
Procedure PrintExpectedError(Error, ExpectedErr: DWORD);
Procedure PrintExpectedErrorEx(Error: DWORD; pExpectedErr : PDWORD);

Function GetModelName(dwModel: DWORD) : String;
Function GetBoxTypeName(wBoxType : Word) : String;

Function  SampleBegin() : Boolean;
Procedure SampleEnd();
Function ScanHardware(wReqBoxType : Word; Var iBoxIndex : Integer; Var IsCrypToken: Boolean): Boolean; overload;
Function ScanHardware(Var iBoxIndex : Integer; Var IsCrypToken: Boolean): Boolean; overload;

Function ROT13fun(const Str: string): string;


Implementation

uses testThread, Main;

{*****************************************************************************
Description:    Print error tag and error description
*****************************************************************************}
procedure PrintError(ErrorText: string);
begin
  if ErrorText = '' then
    Writeln('[ERROR]')
  else
  begin
    Writeln('[ERROR: ', ErrorText, ']');
  end;
end;

{*****************************************************************************
Description:    Translate CBIOS/DO error code to human readable format and print it
*****************************************************************************}
Type
  TErrTextFunc = function(dwError: DWORD; szError: PAnsiChar): DWORD; stdcall;

Procedure PrintErrorCode(Error: DWORD; textfunc : TErrTextFunc);
Var
  S : String;
  szStr: array [0..99] of AnsiChar;
Begin
    szStr[0]:=AnsiChar(0);
    Write(#9);
    //All CBIOS errors please see SmarxOs manual or cbios.h (c/c++)
    textfunc(Error, PAnsiChar(@szStr));
    S := String(szStr);
    if Error = 0 then PrintOK
    else PrintError(S);
    WriteLn;
End;

procedure PrintCBIOSErrorCode(Error: DWORD);
begin
  PrintErrorCode(Error, @CBIOS_ErrorToText);
end;

procedure PrintDOErrorCode(Error: DWORD);
begin
  PrintErrorCode(Error, @TEOS_ErrorToText);
end;

Procedure PrintOK(s: string);
Begin
  Write('[OK');
  if s<>'' then Write(' (' + s + ')]')
  else Write(']');
End;

Procedure PrintOK;
Begin
  PrintOK('');
End;

Procedure _PrintExpectedError(Error, ExpectedErr: DWORD; textfunc : TErrTextFunc);
Var
  szErr1, szErr2: array [0..99] of AnsiChar;
  szTemp : string;
Begin
  Write(#9);
  textfunc(Error, PAnsiChar(@szErr1));
  if Error = ExpectedErr then PrintOK(string(szErr1))
  else
  begin
	  textfunc(ExpectedErr, szErr2);
	  if Error<>0 then szTemp := szErr1 + ' (EXPECTED: ' + string(szErr2) + ')'
	  else szTemp := 'EXPECTED: ' + szErr2;
	  PrintError(szTemp);
  end;
  WriteLn
End;

Procedure PrintExpectedError(Error, ExpectedErr: DWORD);
Begin
  _PrintExpectedError(Error, ExpectedErr, @TEOS_ErrorToText)
End;

Procedure _PrintExpectedErrorEx(Error: LongWord; pExpectedErr : PDWORD; textfunc : TErrTextFunc);
Begin
  if pExpectedErr<>Nil then _PrintExpectedError(Error, pExpectedErr^, textfunc)
	else PrintErrorCode(Error, textfunc);
End;

Procedure PrintExpectedErrorEx(Error: DWORD; pExpectedErr : PDWORD);
Begin
  _PrintExpectedErrorEx(Error, pExpectedErr, @TEOS_ErrorToText)
End;

Function GetModelName(dwModel: DWORD): string;
Begin
  case dwModel of
    CBIOS_CB_MODEL_VERSA: Result:= STR_CB_VERSA;
    CBIOS_CB_MODEL_XS:    Result:= STR_CB_XS;
    CBIOS_CB_MODEL_XL:    Result:= STR_CB_XL;
    CBIOS_CT_MODEL_XS:    Result:= STR_CT_XS;
    CBIOS_CT_MODEL_XL:    Result:= STR_CT_XL;
  else Result:= STR_CB_UNKNOWN
  end
End;

Function GetBoxTypeName(wBoxType : Word) : String;
Begin
	case wBoxType of
    CBIOS_BOX_CBU  : Result := STR_CBU;
    CBIOS_BOX_CBU2 : Result := STR_CBU2;
  else Result := STR_CBU_UNKNOWN
	end
End;

Function _Read : Char;
Begin
  Try
    ReadLn(Result);
  Except
    Result:='0';
  End
End;

Function ScanHardware(wReqBoxType : Word; Var iBoxIndex : Integer; Var IsCrypToken: Boolean): Boolean;
Var
  dwRet, i, a: Longint;
  iNumBoxes, iNumSmarxOSBoxes : Integer;
  fl_DemoSmarxOSBoxFound : Boolean;
  BoxInfo : packed record
       case flBoxInfoAdv:Boolean of
            True : (BoxInfoAdv: CBIOS_BOX_INFO_ADV);
            False: (dwReserve : DWORD; BoxInfo: CBIOS_BOX_INFO);
       end;
  AppInfo: CBIOS_APP_INFO;
  strLabelFmt, strPolicy, strTemp: String;
  c : Char;
  co: TTestThread;
procedure _SetBox;
begin
  iBoxIndex := i;
  IsCrypToken := BoxInfo.BoxInfo.dwModel and $10 <> 0;
end;
Begin
  Result := FALSE;
  iNumSmarxOSBoxes:=0;

    iNumBoxes := CBIOS_ScanBoxes;

    iBoxIndex:=1; iNumSmarxOSBoxes:=0; fl_DemoSmarxOSBoxFound:=False;
    For i:= 1 To iNumBoxes Do
    With BoxInfo.BoxInfo, BoxInfo Do
    Begin
      if CBIOS_IsBoxLockedByOthersI(i) then
      begin
        Continue;
      end;
      flBoxInfoAdv := True;
      BoxInfoAdv.dwStructSize := sizeof(BoxInfoAdv);
      dwRet := CBIOS_GetBoxInfoAdvI(i, @BoxInfoAdv);
      if dwRet = CBIOS_ERR_WRONG_PARAM then
      begin
        flBoxInfoAdv := False;
        dwRet := CBIOS_GetBoxInfoI(i, @BoxInfo);
      end;
      if dwRet <> CBIOS_SUCCESS then continue;

      if flBoxInfoAdv then
      begin
        if BoxInfoAdv.wType >= wReqBoxType then _SetBox;
      end;
      if i = iBoxIndex then _SetBox;

      if 0 = BoxInfo.wCBIOS then
      begin
        // box format error
        Continue;
      end;
      Inc(iNumSmarxOSBoxes);
      if 1 = iNumSmarxOSBoxes then _SetBox;      //Find Demo SmarxOS CRYPTO-BOX:      if not fl_DemoSmarxOSBoxFound then

      for a:=1 to dwApp do
      begin
        dwRet := CBIOS_GetAppInfoI(i, a, @AppInfo);
        if dwRet <> CBIOS_SUCCESS then
        begin
          // GetAppInfo error
          Continue;
        end;
      end;
    End;


    if iNumSmarxOSBoxes = 0 then begin
      //we start a new thread so that the message about disabling the key
      // is issued not immediately after the event occurs,
      // but later for a few seconds:
      co := TTestThread.Create(true);
      co.FreeOnTerminate := true;
      co.Start;
    end;

  Result:=True;
End;

Function ScanHardware(Var iBoxIndex : Integer; Var IsCrypToken: Boolean): Boolean; overload;
Begin
  Result := ScanHardware(CBIOS_BOX_CBU, iBoxIndex, IsCrypToken)
End;


Function SampleBegin() : Boolean;
Var
  dwRet : DWORD;
Begin
  Randomize;
  //Write(':: CBIOS_Startup');
  dwRet := CBIOS_Startup;
  //PrintCBIOSErrorCode(dwRet);
  if (dwRet<>CBIOS_SUCCESS) then
    Result := False
  else
    Result := True;
End;

Procedure SampleEnd();
Var dwRet : DWORD;
Begin
  dwRet := CBIOS_Finish;
End;

procedure ROT13(var Str: string);
const
  OrdBigA = Ord('A');
  OrdBigZ = Ord('Z');
  OrdSmlA = Ord('a');
  OrdSmlZ = Ord('z');
var
  i, o: integer;
begin
  for i := 1 to length(Str) do
  begin
    o := Ord(Str[i]);
    if InRange(o, OrdBigA, OrdBigZ) then
      Str[i] := Chr(OrdBigA + (o - OrdBigA + 13) mod 26)
    else if InRange(o, OrdSmlA, OrdSmlZ) then
      Str[i] := Chr(OrdSmlA + (o - OrdSmlA + 13) mod 26);
  end;
end;

function ROT13fun(const Str: string): string;
begin
  result := Str;
  ROT13(result);
end;

End.
